In Polkadot, we favor privacy preserving parachains being used for actually making purchases, because (a) privacy is a human right, and (b) revealing such private information harms our society, but also because (c) zero-knowledge parachains can produce smaller proof-of-validity blocks, making polkadot more efficient.  In this vein, we consider our parachain model to be a perfect platform for experimentation in privacy preserving parachains.

At the same time, there is a trend in peer-to-peer networks that granting the network operators too much privacy weakens the network.







We're pleased to announce the wire format stabilisation of the primary feature set of our [schnorrkel](https://github.com/w3f/schnorrkel) crate ([docs](https://docs.rs/schnorrkel)), which provides Schnorr signatures, a fast Schnorr DLEQ proof based VRF, hierarchical deterministic key derivation (HDKD), and the safest currently known three round trip Schnorr multi-signature variant.

We want more diverse functionality from signatures used in blockchain applications than from signaturtes used only in TLS, PGP, etc.  Schnorr signatures support more [functionality](https://github.com/sipa/bips/blob/bip-schnorr/bip-schnorr.mediawiki) than ECDSA, and do so more simply, while providing stronger security arguments.  In particular, multi-signatures and threshold signatures are vastly simpler with Schnorr signatures.  We use Schnorr-like signature schemes in schnorrkel for these reason.

We believe schnorrkel strikes an optimal balance between security concerns, including defense-in-depth and miss-use resistance, and the flexibility and diverse functionality wanted from signature schemes in blockchain protocols.   We achieve this balance in basically two ways, first by selecting modern safer cryptographic primitives that extend the strongest primitives in widespread use, and second by assessing the security of the protocols we provide as one cohesive whole, not piecemeal. 

### Primitives

As primitives, we adopt (1) the [Ristretto](https://doc.dalek.rs/curve25519_dalek/ristretto/index.html) implementation of section 7 of Mike Hamburg's [Decaf](https://eprint.iacr.org/2015/673.pdf) paper for our elliptic curve arithmetic, as well as the [merlin](https://doc.dalek.rs/merlin/index.html) limited implementation of Mike Hamberg's [STROBE](http://strobe.sourceforge.io/) for our hashing.

There are many cryptographic protocols that demand a group of large prime order $q$ (aka the number of points or group elements) in which the discrete logarithm problem is hard.  There are two competing design goals in implementing such groups, the [strongest](https://safecurves.cr.yp.to/) elliptic curve choices, like Edwards curves, come with a non-trivial cofactor $h$, meaning the elliptic curve actually has non-prime order $h q$. 

You can adapt simple protocols to a non-trivial cofactor $h$ easily enough by adding a few multiplications by $h$, but they often inherit some malleability and more complex protocols are famously bug prone, including when build from simpler malleabile protocols.  Monero has had multiple serious cofactor bugs, including at least one "infinite inflation" bug in which users could repeatedly multiply their balances' value by the cofactor $h=8$.  In RedDSA, ZCash spent considerable effort excising all malleability from simple Schnorr signtures.  

There are curves like secp256k1 that actually have prime order, making complex protocols safer, but these curve appear [weaker](https://safecurves.cr.yp.to/disc.html) and [serious](https://safecurves.cr.yp.to/ladder.html) [implementation](https://twitter.com/isislovecruft/status/1154184043791040512) [pitfalls](https://safecurves.cr.yp.to/complete.html) [abound](https://safecurves.cr.yp.to/ind.html) for the curves themselves.  And [some](https://mailarchive.ietf.org/arch/msg/tls/Ng1eNx-Q130FWdmYZCER4mlXqMc) people [distrust](https://safecurves.cr.yp.to/rigid.html) the more standard NIST curves like P-256.

In 2015, Mike Hamburg resolved this conflict in his [Decaf](https://eprint.iacr.org/2015/673.pdf) paper, by using an Edwards curve for performance and safety, but using an isogenous prime order curve for serialisation and testing equality.  

[Ristretto](https://doc.dalek.rs/curve25519_dalek/ristretto/index.html) implements roughly section 7 of Mike Hamburg's [Decaf](https://eprint.iacr.org/2015/673.pdf) paper, which handles cofactor 8.  It uses the Ed25519 curve under the hood, so any weaknesses translate into weaknesses in Ed25519.

We employ Ristretto throughout schnorrkel, which reduces malleability, simplifies the analysis of our higher level protocols, and makes it safer to compose schnorrkel protocols with each other and with other similar protocols.

Also, there are minor practical hiccups with the hash functions designed for NIST competitions, but most especially that they create a byte stream oriented interface over a block oriented permutation.  This improves performance of computing MACs on data that arrive in order but piecemeal, but weakens the permutation's natural domain seperation.  We need strong domain seperation in more complex protocols like signatures and NIZKs.  [STROBE](https://strobe.sourceforge.io/) is a strong general purpose symmetric cryptography construction based on Keccak-f(1600), the permutation driving the SHA3 competition winner.  [Merlin](https://doc.dalek.rs/merlin/index.html) is a STROBE scheme for NIZKs, which does almost perfect domain seperation.  

In principle, ristretto and merlin together should let schnorrkel play nicely with other future dalek ecosystem crates, like [bulletproofs](https://github.com/dalek-cryptography/bulletproofs).  We've no current plans to exploit this, but this should simplify weak anonymity parachains analogous to Monero or Mimblewimble. 

### Nonces

Almost all fast signature-like protocols hide the secret key using random nonces aka witnesses.  ECDSA should invoke the system CSPRNG for this.  Yet, there are many bitcoin wallets whose ECDSA does not correctly source randomness and many small devices with weak CSPRNGs, which resulted in the compromise of numerous bitcoin accounts.

We should largely blame bitcoin's original core developers for these wallet developers mistakes.  Ed25519 has always avoided randomness fears during singing by hashing the message with secret key data to generate "derandomized" nonces.  If one Ed25519 key signs the same message twice then both use the same nonce, but this creates no vulnerability in a simple Schnorr signature like Ed25519. 

There are more advanced protocols that cannot be derandomized, including multi-signatures, as well as schemes like batch verification in which derandomization becomes a delicate Fiat-Shamir transform.  We therefore cannot apply derandomization blindly when going beyond simple signatures.

In schnorrkel, we adopt merlin's favoured approachs for nonce generation:  We first apply derandomization but then hash output with some CSPRNG immediately before returning the nonce.  We make the OS's CSPRNG a strong default for this, so it normally provides the security of both approaches.  We must permit user supplied CSPRNGs for test vectors of course.  

There are risks when using our multi-signatures on hardware devices without randomness, but only against your cosigners, and much less than with classical randomized ECDSA signatures.  

### HDKD

Almost all functionality enhancements exploit some limited form of malleability, so their use can make aspects of the signature scheme more fragile.  As such, they should be evaluated carefully, not just alone, but together.  

There are two bizarre signature variations already in heavy usage by current blockchains:  

First, wallets support multiple related addresses using hierarchical deterministic key derivation (HDKD) aka BIP32.  You cannot prevent users from doing HDKD because you cannot remove the public key malleability employed.  We embrace HDKD by providing [clear derivation  routines](https://github.com/w3f/schnorrkel/blob/master/src/derive.rs) in schnorrkel.

As an aside, we also published a crate doing this in [Ed25519](https://github.com/w3f/hd-ed25519), and Tor has a related scheme, but [warn against most variants published for blockchain use](https://forum.web3.foundation/t/key-recovery-attack-on-bip32-ed25519/44).

Second, the ecrecover scheme used with ECDSA on Ethereum saves 32 bytes per signature producing the public key as an output, instead of taking it as an input.  We love saving blocksize, but ecrecover prevents batch verification, making block verification significantly heavier.  We note batch verifications done with ristretto even benefit from [Pippenger multiscalar multiplication](https://github.com/dalek-cryptography/curve25519-dalek/pull/249).  

In ECDSA, there are no known problems with using ecrecover and HDKD together, but using HDKD actually breaks both BLS signatures and the malleable Schnorr signatures that [support ecrecover.](https://www.deadalnix.me/2017/02/17/schnorr-signatures-for-not-so-dummies/).  We therefore use  non-malleable signatures that forbid ecrecover.  

We do not support BLS keys for accounts in polkadot for the same reasons and more.  We note BLS reamins usable in more controlled environments, like consensus protocols.

### Multi-signatures

In schnorrkel, we [implement](https://github.com/w3f/schnorrkel/blob/master/src/musig.rs) the three round trip Schnorr multi-signature scheme from [_"Simple Schnorr Multi-Signatures with Applications to Bitcoin"_](https://eprint.iacr.org/2018/068) by Gregory Maxwell, Andrew Poelstra, Yannick Seurin, and Pieter Wuille.  We enforce this three round trip protocol using "session types", so users cannot deviate form the protocol, except by using only `unsafe` code, rewinding their machine somehow, or similar. 

There are attacks on all previously known two round trip Schnorr multi-signature schemes given in [_"On the Security of Two-Round Multi-Signatures"_](https://eprint.iacr.org/2018/417.pdf) by Manu Drijvers, Kasra Edalatnejad, Bryan Ford, Eike Kiltz, Julian Loss, Gregory Neven, and Igors Stepanovs.  We acknowledge these "concurrency" attacks sound extremely challenging, with two round trip versions actually being secure in various circumstances.  Yet, we do not known any defenses that really appear easily enforceable from library code, so the three round trip version remains preferable for now.

There is one secure two round signature scheme called mBCJ provided in section 5 starting on page 21 of this second article.  Actually deploying mBCJ would require distinct signature verification code, which sets a high bar.  As mBCJ involves some questionable design decisions, we currently view mBCJ merely as a proof that two round trip schemes exist, not actually a deployment target.  

We continue searching for a suitable two-round trip multi-signature scheme.

### Threshold multi-signatures (future work)

We do not currently support threshold signing in our multi-signature scheme.  We expect to do so in future, but so far our priorities lay elsewhere.  We'd also prefer to find a two round trip version before implementing threshold multi-signature.  In fact, there are two several threshold multi-signatures approaches whose costs and benefits depend upon whether we require ne verification code. 

An accountable threshold signature requires your signing key contain either all signers, or else a Merkle root of all signers, so that signatures can identify the signer subset with either a bitfield, or else a Merkle proofs for each signer, respectively.  Any such signature requires bespoke verification code, which again sets a high bar.  We'll do this from the beginning if deploying any scheme that requires new verification code, like mBCJ.

There are non-accountable threshold signing algorithms using Lagrange polynomial interpolation that work for almost any signature scheme, including our existing multi-signature that uses only a standard Schnorr verifier.  Yet, these require a distributed key generation protocol (DKG) using verifiable threshold secret sharing (VSS), which creates one problem:

We cannot permit the threshold shared secret key to be used for anything else, including soft derivations:  If Alice's wallet A has a soft derived wallet B used in a DKG then a threshold of DKG participants can obtain her secret key for B, and derive her secret key for A.  In fact, these DKGs actually create a new shared secret key, on which soft but not hard derivations again make sense, but actually sorting all this out in a miss-use resistant way requires some care.  As an example, we might signers to handle keypairs with combined public keys, and lone combined public keys, never lone secret keys and never create public keys from keypairs.

As an aside, there are now several approaches to doing threshold multi-signatures with ECDSA, like https://eprint.iacr.org/2018/499.pdf and https://eprint.iacr.org/2019/114.pdf and https://eprint.iacr.org/2017/552.pdf, as implemented in [KZen](https://github.com/KZen-networks/multi-party-ecdsa).  These are complex protocols that present interesting targets for attacks, both using the concurrency methods, as well as novel combinations of ecrecover, HDKD, and one another.  I expect ECDSA to eventually crack under the weight of enough such extensions. 

### Verifiable Random Functions (VRF)



Aside from signatures, verifiable random functions are proving to be 

the heavy lifters of 


In cryptography, a pseudo random function (PRF) is a 

randomness cycle from 

We evaluated 

 - Schnorr signatures
 
 
  [related](https://github.com/sipa/bips/blob/bip-schnorr/bip-schnorr.mediawiki) protocols like HDKD, [MuSig](https://eprint.iacr.org/2018/068), and a verifiable random function (VRF).  




There are numerous 








, as well as [related](https://github.com/sipa/bips/blob/bip-schnorr/bip-schnorr.mediawiki) protocols like HDKD, [MuSig](https://eprint.iacr.org/2018/068), and a verifiable random function (VRF).  


  ([related](https://forum.web3.foundation/t/account-signatures-and-keys-in-polkadot/70/3?u=burdges))




, there is a strong interest in migrating to Schnorr signatures because they support more functionality than ECDSA, while providing stronger security arguments.  


, although adaptor signatures and blind signatures also get mentioned in the [Bitcoin Schnorr wishlist](https://github.com/sipa/bips/blob/bip-schnorr/bip-schnorr.mediawiki).




We consider hierarchical deterministic key derivation (HDKD) as used in BIP 32 to be rather dangerous.  We also consider the 

 enough 



As a rule, there are serious risks in 




Schnorr batch verification 

All bBatch verification 


All protocols were adapted to 

Just a few features: 
 - 




Schnorrkel implements Schnorr signature on [Ristretto](https://ristretto.group) compressed Ed25519 points, as well as [related](https://github.com/sipa/bips/blob/bip-schnorr/bip-schnorr.mediawiki) protocols like HDKD, [MuSig](https://eprint.iacr.org/2018/068), and a verifiable random function (VRF).  









# schnorrkel [![](https://img.shields.io/crates/v/schnorrkel.svg)](https://crates.io/crates/schnorrkel) [![](https://docs.rs/schnorrkel/badge.svg)](https://docs.rs/schnorrkel) [![](https://travis-ci.org/w3f/schnorrkel.svg?branch=master)](https://travis-ci.org/w3f/schnorrkel?branch=master)


Schnorrkel implements Schnorr signature on [Ristretto](https://ristretto.group) compressed Ed25519 points, as well as [related](https://github.com/sipa/bips/blob/bip-schnorr/bip-schnorr.mediawiki) protocols like HDKD, [MuSig](https://eprint.iacr.org/2018/068), and a verifiable random function (VRF).  

[Ristretto](https://doc.dalek.rs/curve25519_dalek/ristretto/index.html) implements roughly section 7 of Mike Hamburg's [Decaf](https://eprint.iacr.org/2015/673.pdf) paper to provide the 2-torsion free points of the Ed25519 curve as a prime order group.  ([related](https://forum.web3.foundation/t/account-signatures-and-keys-in-polkadot/70/3?u=burdges))

We employ the [merlin](https://github.com/dalek-cryptography/merlin) strategy of [type specific hashing methods](https://docs.rs/merlin/1.0.3/merlin/struct.Transcript.html) with sound domain separation.  These wrap Mike Hamburg's [STROBE128](https://strobe.sourceforge.io) construction for symmetric cryptography, itself based on Keccak.  

In practice, all our methods consume either a `merlin::Transcript` which developers create handily by feeding data to context specific builders.  We do however also support `&mut merlin::Transcript` like the `merlin` crate prefers.   We shall exploit this in future to adapt schnorrkel to better conform with the dalek ecosystem's zero-knowledge proof tooling. 

We model the VRF itself on ["Making NSEC5 Practical for DNSSEC"](https://eprint.iacr.org/2017/099.pdf) by Dimitrios Papadopoulos, Duane Wessels, Shumon Huque, Moni Naor, Jan Včelák, Leonid Rezyin, andd Sharon Goldberg.  We note the [V(X)EdDSA signature scheme](https://www.signal.org/docs/specifications/xeddsa/#vxeddsa) by Trevor Perrin at is basically identical to the NSEC5 construction.  Also, the VRF supports individual signers merging numerous VRF outputs created with the same keypair, which parallels the "DLEQ Proofs" and "Batching the Proofs" sections of ["Privacy Pass - The Math"](https://blog.cloudflare.com/privacy-pass-the-math/#dleqproofs) by Alex Davidson, and ["Privacy Pass: Bypassing Internet Challenges Anonymously"](https://www.petsymposium.org/2018/files/papers/issue3/popets-2018-0026.pdf)
by Alex Davidson, Ian Goldberg, Nick Sullivan, George Tankersley, and Filippo Valsorda.

Aside from some naive sequential VRF construction, we currently only support the three-round [MuSig](https://eprint.iacr.org/2018/068) for Schnorr multi-signatures, due to all other Schnorr multi-signatures being somewhat broken.  In future, we should develop secure schemes like mBCJ from section 5.1 starting page 21 of https://eprint.iacr.org/2018/417 however mBCJ itself works by proof-of-possession, while a [delinearized](http://crypto.stanford.edu/~dabo/pubs/abstracts/aggsurvey.html) variant sounds more applicable.


There are partial bindings for [C](https://github.com/Warchant/sr25519-crust), [JavaScript](https://github.com/paritytech/schnorrkel-js/), and [Python](https://gitlab.com/kauriid/schnorrpy/) as well.




